@import url('https://fonts.googleapis.com/css2?family=Inter:wght@400;500;700;900&display=swap');

body {
    font-family: 'Inter', sans-serif;
}

a {
    text-decoration: none;
}

table.docutils.align-default {
    min-width: 100%;
}
