name: Basic
original_tempo: 180
bars: {{bars}}
clips:
  - name: bass
    path: clips/bass.wav
parts:
  - name: backbeat
    bars: {{bars}}
    clips:
      - name: backbeat
        bars: {{bars}}
mix:
  - name: intro0
    parts:
      - name: backbeat
